using System;
using TDDExample.GuessNumber;

namespace GuessNumber
{
    public class GameEngine
    {
        private readonly GameController gameController;
        private readonly InputValidator validator;
        private Action onExit;
        private bool stop;

        public GameEngine(GameController gameController, InputValidator validator)
        {
            this.gameController = gameController;
            this.validator = validator;
            gameController.GameClear += game_GameClear;
            gameController.GameOver += game_GameOver;
        }

        private void game_GameOver()
        {
            Exit(() => Console.WriteLine("Game Over! The answer is {0}!", gameController.Answer));
        }

        private void game_GameClear()
        {
            Exit(() => Console.WriteLine("Bingo! You win the game!"));
        }

        private void Exit(Action onExitAction)
        {
            stop = true;
            onExit = onExitAction;
        }

        public void Run()
        {
            while (!stop)
            {
                string input = Console.ReadLine();
                if (!validator.Validate(input))
                {
                    Console.WriteLine("Unknown input format. Please input numbers like this: 1 2 3 4");
                    continue;
                }

                int[] guessNum = GetGuessNums(input);
                Console.WriteLine(gameController.Guess(guessNum));

                PrintGuessHistory();
            }
            onExit();
        }

        private void PrintGuessHistory()
        {
            Console.WriteLine("---------------Guess Attempts-------------------");
            Console.Write(gameController.GetGuessHistory());
            Console.WriteLine("------------------------------------------------");
            Console.WriteLine();
        }

        private static int[] GetGuessNums(string input)
        {
            string[] numbers = input.Split(new[] {' '}, StringSplitOptions.RemoveEmptyEntries);
            var result = new int[numbers.Length];
            for (int i = 0; i < numbers.Length; i++)
            {
                result[i] = Convert.ToInt32(numbers[i]);
            }
            return result;
        }
    }
}