﻿using System;
using Autofac.Builder;
using TDDExample.GuessNumber;

namespace GuessNumber
{
    internal class Program
    {
        private static readonly ContainerBuilder builder = new ContainerBuilder();
        private static void Main(string[] args)
        {
            SetupDependencies();
            PrintUsage();
            StartGame();
        }

        private static void SetupDependencies()
        {
            builder.Register<AnswerGenerator>().As<IAnswerGenerator>();
            builder.Register<GameHistory>().As<IGameHistory>();
            builder.Register<RandomIntGenerator>().As<IRandomIntGenerator>();
            builder.Register<InputValidator>();
            builder.Register<Game>().As<IGame>();
            builder.Register<GameController>();
            builder.Register<GameEngine>();
        }

        private static void PrintUsage()
        {
            Console.WriteLine("Guess number v1.0");
        }

        private static void StartGame()
        {
            using(var container = builder.Build())
            {
                var engine = container.Resolve<GameEngine>();
                engine.Run();
            }
        }
    }
}