using System.Collections.Generic;
using Moq;
using NUnit.Framework;

namespace TDDExample.GuessNumber.UnitTests
{
    [TestFixture]
    public class TestAnswerGenerator
    {
        [Test]
        public void should_be_able_to_generate_0_and_9()
        {
            var numbers = new[] {0, 1, 8, 9};
            int index = 0;
            var mock = new Mock<IRandomIntGenerator>();
            mock.Setup(g => g.Next()).Returns(() => numbers[index]).Callback(() => index++);

            IAnswerGenerator generator = new AnswerGenerator(mock.Object);
            int[] answer = generator.Generate();
            Assert.That(answer, Is.EquivalentTo(new int[] {0, 1, 8, 9}));
        }

        [Test]
        public void should_generate_non_repeatable_numbers()
        {
            var numbers = new[] {1, 2, 3, 3, 4, 5, 6};
            int index = 0;
            var mock = new Mock<IRandomIntGenerator>();
            mock.Setup(g => g.Next()).Returns(() => numbers[index]).Callback(() => index++);

            IAnswerGenerator generator = new AnswerGenerator(mock.Object);
            int[] answer = generator.Generate();
            Assert.That(answer, Is.EquivalentTo(new int[] {1, 2, 3, 4}));

            mock.Verify(g => g.Next(), Times.Exactly(5));
        }

        [Test]
        public void should_generate_numbers_between_zero_and_nine()
        {
            var mock = new Mock<IRandomIntGenerator>();
            mock.Setup(g => g.Next()).Returns(10);

            IAnswerGenerator generator = new AnswerGenerator(mock.Object);
            Assert.Throws<RandomNumberOutOfRangeException>(() => generator.Generate());
        }
    }
}