﻿using Moq;
using NUnit.Framework;

namespace TDDExample.GuessNumber.UnitTests
{
    [TestFixture]
    public class TestGame
    {
        [SetUp]
        public void Setup()
        {
            var mockAnswerGenerator = new Mock<IAnswerGenerator>();
            mockAnswerGenerator.Setup(generator => generator.Generate()).Returns(new[] {1, 2, 3, 4});

            game = new Game(mockAnswerGenerator.Object);
        }

        private Game game;
       
        [Test]
        public void should_get_the_answer_string()
        {
            Assert.That(game.Answer, Is.EqualTo("1 2 3 4"));
        }
      

        [Test]
        public void should_return_0A0B_when_no_number_is_correct()
        {
            Assert.That(game.Guess(new[] {5, 6, 7, 8}), Is.EqualTo("0A0B"));
        }

        [Test]
        public void should_return_0A2B_when_two_numbers_are_correct_but_positions_are_not_correct()
        {
            Assert.That(game.Guess(new[] {3, 4, 5, 6}), Is.EqualTo("0A2B"));
        }

        [Test]
        public void should_return_1A0B_when_one_number_is_correct_and_position_is_correct_too()
        {
            Assert.That(game.Guess(new[] {1, 5, 6, 7}), Is.EqualTo("1A0B"));
        }

        [Test]
        public void should_return_2A2B_when_two_numbers_are_pisition_correct_and_two_are_nunmber_correct()
        {
            Assert.That(game.Guess(new[] {1, 2, 4, 3}), Is.EqualTo("2A2B"));
        }

        [Test]
        public void should_return_4A0B_when_all_numbers_are_pisition_correct()
        {
            Assert.That(game.Guess(new[] {1, 2, 3, 4}), Is.EqualTo("4A0B"));
        }

    }
}