﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Moq;
using NUnit.Framework;

namespace TDDExample.GuessNumber.UnitTests
{
    [TestFixture]
    public class TestGameController
    {
        public interface IGameObserver
        {
            void GameOver();
            void GameClear();
        }

        private Mock<IGameHistory> mockHistory;
        private Mock<IGame> mockGame;
        private GameController controller;

        [SetUp]
        public void Setup()
        {
            mockHistory = new Mock<IGameHistory>();
            mockGame = new Mock<IGame>();
            controller = new GameController(mockGame.Object, mockHistory.Object);
         
        }

        [Test]
        public void should_record_guess_history()
        {
            mockGame.Setup(game => game.Guess(It.IsAny<int[]>())).Returns("0A0B");
           
            controller.Guess(new[] { 5, 6, 7, 8 });
            
            mockHistory.Verify(h => h.Add(new[] { 5, 6, 7, 8 }, "0A0B"));
        }

        [Test]
        public void should_fail_game_when_guess_six_times_and_still_wrong()
        {
            var wrongGuess = new[] { 5, 6, 7, 8 };
            var mockObserver = new Mock<IGameObserver>();
            mockGame.Setup(game => game.Guess(wrongGuess)).Returns("0A0B");
            controller.GameOver += mockObserver.Object.GameOver;
         
            for (int i = 0; i < 5; i++)
            {
                controller.Guess(wrongGuess);
            }
            
            mockObserver.Verify(m => m.GameOver(), Times.Never());
            
            controller.Guess(wrongGuess);
            
            mockObserver.Verify(m => m.GameOver(), Times.Exactly(1));
            mockGame.Verify(m=>m.Guess(wrongGuess), Times.Exactly(6));
        }

        [Test]
        public void should_win_when_guess_correct()
        {
            var correctAnswer = new[] { 1, 2, 3, 4 };
            var mockObserver = new Mock<IGameObserver>();
            mockGame.Setup(game => game.Guess(correctAnswer)).Returns("4A0B");
            controller.GameClear += mockObserver.Object.GameClear;

            controller.Guess(correctAnswer);
            
            mockObserver.Verify(m => m.GameClear(), Times.Exactly(1));
            mockGame.Verify(m => m.Guess(correctAnswer), Times.Exactly(1));

        }
    }
}
