namespace TDDExample.GuessNumber
{
    public class GameController
    {
        public delegate void GameEventHandler();

        private const string CorrectGuess = "4A0B";
        private const int MaxGuessTimes = 6;
        private readonly IGame game;
        private readonly IGameHistory history;
        private int guessTimes;


        public GameController(IGame game, IGameHistory history)
        {
            this.game = game;
            this.history = history;
        }

        public string Answer
        {
            get { return game.Answer; }
        }

        public string Guess(int[] guess)
        {
            guessTimes++;

            string result = game.Guess(guess);
            RecordGuess(guess, result);

            if (IsGameClear(result))
            {
                OnGameClear();
            }
            else if (CanContinueGuess())
            {
                OnGameOver();
            }
            return result;
        }

        public event GameEventHandler GameOver;
        public event GameEventHandler GameClear;

        private static bool IsGameClear(string result)
        {
            return result == CorrectGuess;
        }


        private bool CanContinueGuess()
        {
            return guessTimes >= MaxGuessTimes;
        }

        protected void OnGameClear()
        {
            if (GameClear != null)
            {
                GameClear();
            }
        }

        protected void OnGameOver()
        {
            if (GameOver != null)
            {
                GameOver();
            }
        }

        public string GetGuessHistory()
        {
            return history.GetAll();
        }

        private void RecordGuess(int[] guess, string result)
        {
            if (history != null)
            {
                history.Add(guess, result);
            }
        }
    }
}