﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TDDExample.GuessNumber
{
    public class InputValidator
    {
        private const int InputLength = 4;
        private const int MaxNum = 9;
        private const int MinNum = 0;

        public bool Validate(string input)
        {
            string[] numbers = input.Split(new[] {' '}, StringSplitOptions.RemoveEmptyEntries);
            return IsInputLengthCorrect(numbers)
                   && IsInputAreAllInteger(numbers)
                   && IsInputAreAllBetweenZeroAndNine(numbers)
                   && IsUnique(numbers);
        }

        private static bool IsInputAreAllBetweenZeroAndNine(IEnumerable<string> numbers)
        {
            return numbers.All(numStr => Convert.ToInt32((string) numStr).IsBetween(MinNum, MaxNum));
        }

        private static bool IsInputLengthCorrect(ICollection<string> numbers)
        {
            bool correct = numbers.Count == InputLength;
            return correct;
        }

        private static bool IsInputAreAllInteger(IEnumerable<string> numbers)
        {
            int num;
            return numbers.All(number => int.TryParse(number, out num));
        }

        private static bool IsUnique(IEnumerable<string> numbers)
        {
            return numbers.Distinct().Count() == InputLength;
        }
    }
}